﻿<#
.SYNOPSIS
Script to launch or exit a pre-defined list of applications.
.DESCRIPTION
Script allows to launch or exit a pre-defined list of applications.  You define the applications in a text file, one line per program, with the exact path to the executable. 
Wrap the full absolute file path in double quotes. If the program requires any arguments, add them too. Example:

If you put these two lines in the text file, these applications will be launched in this order.

"C:\Program Files (x86)\Elecraft\W2 Meter\W2.exe"
"C:\Program Files (x86)\PstRotatorAz\PstRotatorAz.exe"
"C:\Program Files (x86)\wintelnetx_4_22\wintelnetx.exe" -start ini\MySkimmer_plus_W9PA_to_localhost23.ini
"C:\Program Files\TightVNC\tvnviewer.exe" -optionsfile=C:\Users\andre\Documents\RF2K+VNCConnection.vnc
"C:\Program Files (x86)\N1MMLogger+\N1MMLogger.net.exe" UserDir="C:\Users\andre\Documents\N1MM Logger+" INI="C:\Users\andre\Documents\N1MM Logger+\N1MM Logger.ini"

You can use the special parameter /MIN to signal that the window should be minized. Add this parameter as an argument on the same line.

If you must tell the script to wait for a few seconds, add this on its own line (15 s in this example):

DELAY_IN_SECONDS 15

History:
09/05/2018: V1: main functionality
10/22/2018: V2: added delay that can be cancelled with key

.PARAMETER applicationsFilePath 
The full file path to a text file that lists the files to be launched (or closed).
.PARAMETER launchOrExit
This parameter controls whether the applications in the list are being launched or closed.
.EXAMPLE 
Launch all applications that are defined in the file hamapps1.txt, located next to this script:
AppAutomation.ps1 -launchOrExit $true -file .\hamapps1.txt
.EXAMPLE
Close all applications that are defined in the file hamapps1.txt, located next to this script:
AppAutomation.ps1 -launchOrExit $false -file .\hamapps1.txt
#>
Param(
    [Parameter(Mandatory=$true)][string]$applicationsFilePath="$PSScriptRoot\hamapps1.txt"  # The file that defines the list of executables to launch or close
    , [Parameter(Mandatory=$true)][boolean]$launchOrExit=$true)             # $true for launch, $false for close the applications

function Set-WindowStyle {
param(
    [Parameter()]
    [ValidateSet('FORCEMINIMIZE', 'HIDE', 'MAXIMIZE', 'MINIMIZE', 'RESTORE', 
                 'SHOW', 'SHOWDEFAULT', 'SHOWMAXIMIZED', 'SHOWMINIMIZED', 
                 'SHOWMINNOACTIVE', 'SHOWNA', 'SHOWNOACTIVATE', 'SHOWNORMAL')]
    $Style = 'SHOW',
    [Parameter()]
    $MainWindowHandle = (Get-Process -Id $pid).MainWindowHandle
)
    $WindowStates = @{
        FORCEMINIMIZE   = 11; HIDE            = 0
        MAXIMIZE        = 3;  MINIMIZE        = 6
        RESTORE         = 9;  SHOW            = 5
        SHOWDEFAULT     = 10; SHOWMAXIMIZED   = 3
        SHOWMINIMIZED   = 2;  SHOWMINNOACTIVE = 7
        SHOWNA          = 8;  SHOWNOACTIVATE  = 4
        SHOWNORMAL      = 1
    }
    Write-Verbose ("Set Window Style {1} on handle {0}" -f $MainWindowHandle, $($WindowStates[$style]))

    $Win32ShowWindowAsync = Add-Type –memberDefinition @” 
    [DllImport("user32.dll")] 
    public static extern bool ShowWindowAsync(IntPtr hWnd, int nCmdShow);
“@ -name “Win32ShowWindowAsync” -namespace Win32Functions –passThru

    $Win32ShowWindowAsync::ShowWindowAsync($MainWindowHandle, $WindowStates[$Style]) | Out-Null
}

function ParseCommandLine([string] $cmdLine)
{
    $returnArray = @()
    [int32]$delaySeconds = 0

    if($cmdLine.IndexOf('DELAY_IN_SECONDS') -gt -1)
    {
        $splitCmdLine = $cmdLine.Split(' ')
        if($splitCmdLine.Count -eq 2)
        {
            if($false -eq [int32]::TryParse($splitCmdLine[1], [ref]$delaySeconds))
            {
                Write-Error "parsing error of $cmdLine"
            }
        }
        else
        {
                Write-Error "parsing error of $cmdLine"
        }
    }
    else
    {
        # find file end is after 2nd double quote
        [string] $executableFullFilePath = ""
        $firstQuoteIndex = $cmdLine.IndexOf('"')
        if($firstQuoteIndex -gt -1) {
            $secondQuoteIndex = $cmdLine.IndexOf('"', $firstQuoteIndex + 1)
        }
    
        if($secondQuoteIndex -gt -1) {
            $secondQuoteIndex = $cmdLine.IndexOf('"', $firstQuoteIndex + 1)
            $executableFullFilePath = $cmdLine.Substring($firstQuoteIndex + 1, $secondQuoteIndex - $firstQuoteIndex - 1)
        }

        if([string]::IsNullOrEmpty($executableFullFilePath)) {
            Write-Error "parsing error of $cmdLine"
        }    

        [string] $executableFolderPath = [System.Io.Path]::GetDirectoryName($executableFullFilePath)
        [string] $executableFileName = [System.Io.Path]::GetFileName($executableFullFilePath)
        [string] $executableFileNameWithoutExtension = [System.Io.Path]::GetFileNameWithoutExtension($executableFullFilePath)
        $arguments = @()
        if($cmdLine.Length - $secondQuoteIndex - 1 -gt 0)
        {
            [string] $executableArgsString = $cmdLine.Substring($secondQuoteIndex + 1, $cmdLine.Length - $secondQuoteIndex - 1)
            $arguments = @($executableArgsString.Split(' ').Where({$_.Trim() -ne ' '}))
        }

        [System.Collections.ArrayList]$realArguments = $arguments

        $shouldMinimize = $false
     
        if($arguments -contains '/MIN') {
            $realArguments.Remove("/MIN")
            $shouldMinimize = $true
        }
    }

    $returnArray = @(
        $executableFolderPath,
        $executableFileName,
        $executableFileNameWithoutExtension,
        @($realArguments),
        $shouldMinimize,
        [TimeSpan]::FromSeconds($delaySeconds)
    )

    return ,$returnArray
}

function LaunchProgramsInFile([string] $file)
{
    $apps = @(Get-Content $file | Where-Object {$_ -match '^[^#;]'})
    for ($i=0; $i -lt $apps.Count; $i++) {
        $launchParameters = ParseCommandLine $apps[$i]
        PrintLaunchParameters $launchParameters
        if($launchParameters[5] -gt [Timespan]::Zero) # delay in seconds
        {
			Pause-HostInSeconds $launchParameters[5].Seconds
        }
        else
        {
            LaunchProgram $launchParameters
            if($launchParameters[4] -eq $true) {
                MinimizeProgram $launchParameters
            }
        }
    }
}

function LaunchProgram($launchParameters)
{
    $executableFolderPath = $launchParameters[0]
    $executableFileName = $launchParameters[1]
    $executableFileNameWithoutExtension = $launchParameters[2]
    $realArguments = $launchParameters[3]
    $shouldMinimize = $launchParameters[4]

    $process = Get-Process | Where-Object {$_.ProcessName -eq "$executableFileNameWithoutExtension"}
    if($process -ne $null) {
        Write-Host "$executableFileName already running..."
        return
    }

    Write-Host "launching $executableFileName..."

    invoke-command -scriptblock {
		$loc = Get-Location
        Set-Location $executableFolderPath
        & .\$executableFileName $realArguments
		Set-Location $loc 
    }
}

function MinimizeProgram($launchParameters)
{
    $executableFolderPath = $launchParameters[0]
    $executableFileName = $launchParameters[1]
    $executableFileNameWithoutExtension = $launchParameters[2]
    $realArguments = $launchParameters[3]
    $shouldMinimize = $launchParameters[4]

    if($shouldMinimize){
        Write-Host "minimizing $executableFileName..."

        $process = Get-Process | Where-Object {$_.ProcessName -eq "$executableFileNameWithoutExtension"}
        if($process -ne $null) {
            $process.MainWindowHandle | foreach { Set-WindowStyle MINIMIZE $_}
        }
    }
}

function CloseProgramsInFile([string] $file)
{
    $apps = Get-Content $file | Where-Object {$_ -match '^[^#;]'}
    for ($i=$apps.Count - 1; $i -gt -1; $i--) {
        $launchParameters = ParseCommandLine $apps[$i]
        PrintLaunchParameters $launchParameters
        CloseProgram $launchParameters
    }
}

function CloseProgram($launchParameters)
{
    $executableFolderPath = $launchParameters[0]
    $executableFileName = $launchParameters[1]
    $executableFileNameWithoutExtension = $launchParameters[2]
    $realArguments = $launchParameters[3]
    $shouldMinimize = $launchParameters[4]

    Write-Host "closing $executableFileName..."

    $process = Get-Process | Where-Object {$_.ProcessName -eq "$executableFileNameWithoutExtension"}
    if($process -ne $null) {
        # $process.MainWindowHandle | foreach { Set-WindowStyle MAXIMIZE $_}
        $process | % { $_.CloseMainWindow() | Out-Null}
    }
}

function PrintLaunchParameters($launchParameters)
{
    Write-Verbose '********************'
    Write-Verbose ($launchParameters -join "`r`n")
    Write-Verbose '********************'
}

function Pause-HostInSeconds
{
    param(
            $Delay = 1
         )

    Write-Host "Sleeping for $($launchParameters[5].Seconds) seconds or hit ENTER..."
		 
    $counter = 0;
    While(!$host.UI.RawUI.KeyAvailable -and ($counter++ -lt $Delay))
    {
        [Threading.Thread]::Sleep(1000)
    }
}

try
{
    $oldVerbose = $VerbosePreference 
    
    # for better debugging, enable this line
    # $VerbosePreference = "continue" 

    if($launchOrExit -eq $true)
    {
        LaunchProgramsInFile $applicationsFilePath
    }
    else
    {
        CloseProgramsInFile $applicationsFilePath
    }

    Read-Host 'done. Hit Enter' | Out-Null
}
finally
{
    $VerbosePreference = $oldVerbose
}